/*
 * Mintaprogramok/Feladatok
 * Projekt: Cimjegyzek
 * Csomag: gui
 * MainFrame.java
 *
 * Angster Erzsbet: OO tervezs s programozs, Java II. ktet
 * 2002.09.01.
 */

package gui;
import db.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class MainFrame extends JFrame implements
                       ActionListener, KeyListener {
  // Kontner, a "szemelyek.dat" llomnyt kezeli:
  private Szemelyek szemelyek;

  private Container cp = getContentPane();
  private JMenuItem miHozzaad, miModosit, miTorol, miKilep;
  private JMenuItem miSegitseg, miNevjegy;

  private JButton btHozzaad, btModosit, btTorol;
  private NevjegyDialog diaNevjegy = null;
  private SegitsegDialog diaSegitseg = null ;
  private SzemelyDialog diaSzemely = null;

  private JList lstSzemelyek;

  public MainFrame() {
    // GUI felptse:
    setTitle("Cmjegyzk");
    setBounds(200,200,300,200);
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    JMenuBar mb;
    JMenu mFajl, mSegitseg;
    setJMenuBar(mb = new JMenuBar());
    mb.add(mFajl = new JMenu("Fjl"));
    mFajl.setMnemonic('F');
    mFajl.add(miHozzaad = new JMenuItem("Hozzad",'H'));
    mFajl.add(miModosit = new JMenuItem("Mdost",'M'));
    mFajl.add(miTorol = new JMenuItem("Trl",'T'));
    mFajl.addSeparator();
    mFajl.add(miKilep = new JMenuItem("Kilp",'K'));
    mb.add(mSegitseg = new JMenu("Sg"));
    mSegitseg.setMnemonic('S');
    mSegitseg.add(miSegitseg = new JMenuItem("Segtsg",'S'));
    mSegitseg.addSeparator();
    mSegitseg.add(miNevjegy = new JMenuItem("Nvjegy",'N'));

    miHozzaad.addActionListener(this);
    miModosit.addActionListener(this);
    miTorol.addActionListener(this);
    miKilep.addActionListener(this);
    miSegitseg.addActionListener(this);
    miNevjegy.addActionListener(this);

    JPanel pnButtons;
    cp.add(pnButtons = new JPanel(),"South");
    pnButtons.add(btHozzaad = new JButton("Hozzad"));
    pnButtons.add(btModosit = new JButton("Mdost"));
    pnButtons.add(btTorol = new JButton("Trl"));
    btHozzaad.setMnemonic('H');
    btModosit.setMnemonic('M');
    btTorol.setMnemonic('T');
    btHozzaad.addActionListener(this);
    btModosit.addActionListener(this);
    btTorol.addActionListener(this);

    // File kapcsolat kiptse, a lista elksztse:
    try {
      szemelyek = new Szemelyek("adatok/szemelyek.dat");
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this,
        "Hiba a fjl megnyitsakor!","",JOptionPane.ERROR_MESSAGE);
      System.exit(0);
    }

    // A lista adatmodelljt a kulcsbejegyzsekkel inicializljuk:
    lstSzemelyek = new JList(szemelyek.keyEntries);
    // Rtesszk egy grgetpanelre:
    cp.add(new JScrollPane(lstSzemelyek),"Center");
    // Csak egy ttelt lehet kivlasztani egyszerre:
    lstSzemelyek.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // Kezdetben az els elem a kivlasztott, ha van ilyen:
    if (lstSzemelyek.getModel().getSize()!=0)
      lstSzemelyek.setSelectedIndex(0);

    // A felvitel, a mdosts s a trls billentykre is hvhat:
    this.addKeyListener(this);
    show();
    lstSzemelyek.requestFocus();

    // A dialgusokat ltrehozzuk, majd igny szerint mutogatjuk ket:
    diaNevjegy = new NevjegyDialog(this);
    diaSegitseg = new SegitsegDialog(this);
    diaSzemely = new SzemelyDialog(this);
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  // j szemly adatainak felvitele:
  public void hozzaad() {
    Szemely szemely = new Szemely();
    // Egy szemly objektum sszelltsa:
    if (diaSzemely.showDialog(szemely)) {
      try {
        // A szemly eltrolsa:
        szemelyek.addSzemely(szemely);
        // A lista adatmodelljnek frisstse:
        lstSzemelyek.setListData(szemelyek.keyEntries);
      }
      catch (IOException ex) {
        JOptionPane.showMessageDialog(this,
          "Hiba a fjl rsakor!","",JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  // Mr ltez szemly adatainak mdostsa:
  public void modosit() {
    // index a kivlasztott listaelem indexe:
    int index = lstSzemelyek.getSelectedIndex();
    // Ha nem vlasztottak ki semmit, nem mdostunk:
    if (index == -1)
      return;
    try {
      // Elkrjk a troltl a nvsorban az indexedik szemlyt:
      Szemely szemely = szemelyek.getSzemely(index);
      // Felknljuk a szmlyt mdostsra:
      if (diaSzemely.showDialog(szemely)) {
        try {
          // Kicserljk a trolban az index. szemlyt.
          // Ezutn lehet, hogy ms lesz az indexe:
          szemelyek.replaceSzemely(szemely,index);
          // A lista adatmodelljnek frisstse:
          lstSzemelyek.setListData(szemelyek.keyEntries);
          // Ugyanaz a sorszm elem marad kivlasztva:
          lstSzemelyek.setSelectedIndex(index);
        }
        catch (IOException ex) {
          JOptionPane.showMessageDialog(this,
            "Hiba a fjl rsakor!","",JOptionPane.ERROR_MESSAGE);
        }
      }
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog(this,
        "Hiba a fjl rsakor!","",JOptionPane.ERROR_MESSAGE);
    }
  }

  // Mr ltez szemly trlse
  public void torol() {
    // Az index. szemlyt akarjk ttlni:
    int index = lstSzemelyek.getSelectedIndex();
    if (index == -1)
      return;
    if (tenyleg("Biztosan trlni akarja?")) {
      try {
        // Kitrljk a szemlyt a trolbl:
        szemelyek.removeSzemely(index);
        // A lista adatmodelljnek frisstse:
        lstSzemelyek.setListData(szemelyek.keyEntries);
      }
      catch (IOException ex) {
        JOptionPane.showMessageDialog(this,
          "Hiba a fjl rsakor!","",JOptionPane.ERROR_MESSAGE);
      }
      // A kivlasztst szksg esetn mdostjuk.
      // Ha a lista nem res, s az utols elemt trltk,
      // akkor eggyel elrbb llunk.
      int size = lstSzemelyek.getModel().getSize();
      if (size > 0) {
        if (index==size)
          index--;
        lstSzemelyek.setSelectedIndex(index);
      }
    }
  }

  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    if (src==miNevjegy) {
      diaNevjegy.setVisible(true);
    }
    else if (src==miSegitseg) {
      diaSegitseg.setVisible(true);
    }
    else if (src==btHozzaad || src==miHozzaad) {
      hozzaad();
    }
    else if (src==btModosit || src==lstSzemelyek) {
      modosit();
    }
    else if (src==btTorol) {
      torol();
    }
    else if (src==miKilep) {
      try {
        szemelyek.close();
      }
      catch (IOException ex) {
        System.out.println(ex);
      }
      System.exit(0);
    }
    lstSzemelyek.requestFocus();
  }

  public void keyTyped(KeyEvent e) {
  }

  // INSERT-re felvitel, ENTER-er mdosts, DELETE billentyre trls:
  public void keyPressed(KeyEvent e) {
    if (e.getKeyCode()==KeyEvent.VK_INSERT)
      hozzaad();
    else if (e.getKeyCode()==KeyEvent.VK_ENTER)
      modosit();
    else if (e.getKeyCode()==KeyEvent.VK_DELETE)
      torol();
  }

  public void keyReleased(KeyEvent e) {
  }

  // Az message krdsre Igen/Nem vlaszt vr. true, ha igen:
  boolean tenyleg(String message) {
    Object[] opciok = {"Igen","Nem"};
    int valasz = JOptionPane.showOptionDialog(this,
      message,"",JOptionPane.DEFAULT_OPTION,
      JOptionPane.WARNING_MESSAGE,null,opciok,opciok[1]);
    return valasz==0;
  }

  private void jbInit() throws Exception {
  }
}
